let rootUrl = 'http://192.168.2.109:8080/';
let dmnApi = rootUrl + 'api/dmn/';

/**
 * Global definition for verification results
 * @type {VerifierResultSet}
 */
let verifierResults = {};

/**
 *
 * @type {Array.<VerificationType>}
 */
let types = [];

let $verifierTypes;

// load types
$.ajax({
  url: dmnApi + 'verification/type',
  type: 'GET',
  contentType: 'text/xml',
  success: function (data) {
    types = data;
    // sort types by classification and name
    types.sort(function (
        /** VerificationType */ firstEl, /** VerificationType */ secondEl) {
      if (firstEl.classification.name
          === secondEl.classification.name) {
        if (firstEl.niceName === secondEl.niceName) {
          return 0;
        }
        return firstEl.niceName < secondEl.niceName ? -1 : 1;
      } else if (firstEl.classification.name
          < secondEl.classification.name) {
        return -1;
      } else {
        return 1;
      }
    });
    $verifierTypes = renderTypeOptions();
  }
});

function cleanDmnVerifierRoot() {
  let $root = $('#root-dmn-verifier').empty();
  let $header = $(`      
      <div id="dmn-verifier-header">
  `);
  // add types
  $header.append($verifierTypes);
  // Add Button
  $header.append($(`
    <button class="clickable dmn-verifier-header-item" id="dmn-button-verify" onClick="checkVerifications()">
    Verify</button>
  `));
  // add header to root
  $root.append($header);
  $root.append($(`<div id="dmn-verifier-content">`));
  return $root;
}

function checkVerifications() {
  cleanDmnVerifierRoot();
  getVerifications();
}

function getVerifications() {
  dmnModeler.saveXML({format: true}, function (err, xml) {
    if (err) {
      return console.log(
          'There is an error in requesting the dmn verifications.. ' + err);
    } else {
      console.log('Requesting dmn verifications..');
    }
    let apiPath = dmnApi + 'verification';
    // check, if a verifier is preselected
    if ($verifierTypes[0].selectedOptions[0].value !== 'all') {
      apiPath += "/type/" + $verifierTypes[0].selectedOptions[0].value;
    }
    $.ajax({
      url: apiPath,
      type: 'POST',
      contentType: 'text/xml',
      data: xml,

      success: function (data) {
        console.log('successful dmn verification request:');
        console.log(data);
        verifierResults = data;
        // sort the list 1. classification, 2. size of entries
        verifierResults.verifier.sort(function (
            /** VerifierResult */ firstEl, /** VerifierResult */ secondEl) {
          if (firstEl.type.classification.name
              === secondEl.type.classification.name) {
            if (firstEl.size === secondEl.size) {
              return firstEl.type.name < secondEl.type.name ? -1 : 1;
            } else if (firstEl.size < secondEl.size) {
              return 1;
            } else {
              return -1;
            }
          } else if (firstEl.type.classification.name
              < secondEl.type.classification.name) {
            return -1;
          } else {
            return 1;
          }
        });
        renderDmnVerifierOptions();
      }
    });
  });
}

function renderTypeOptions() {
  let $select = $(`
    <select name="verifier" id="dmn-verifier-types"
      class="dmn-verifier-header-item dmn-verifier-select clickable">
  `);
  $select.append($(`<option value="all">All verifier</option>`));
  let currentOpt = '';
  let $curGroup;
  for (let i = 0; i < types.length; i++) {
    let bolNewGroup = currentOpt
        !== types[i].classification.niceName;
    if (bolNewGroup) {
      if (currentOpt === currentOpt) {
        $select.append($curGroup);
      }
      $curGroup = ($(
          `<optgroup label="${types[i].classification.niceName}">
      `));
      currentOpt = types[i].classification.niceName;
    }
    $curGroup.append($(`
      <option value="${types[i].name}">${types[i].niceName}</option>`
    ));

  }
  $select.append($curGroup);
  return $select;
}

function renderDmnVerifierOptions() {
  let $header = $('#dmn-verifier-header');
  let $select = $(`
    <select name="verifier" id="verifier"
      class="dmn-verifier-header-item dmn-verifier-select clickable"
      onchange="renderVerifierResult(this)">
  `);
  $select.append($(`<option>Select a verifier</option>`));
  let currentOpt = '';
  let $curGroup;
  for (let i = 0; i < verifierResults.verifier.length; i++) {
    let verifier = verifierResults.verifier[i];
    let bolNewGroup = currentOpt
        !== verifier.type.classification.niceName;
    if (bolNewGroup) {
      if (currentOpt === currentOpt) {
        $select.append($curGroup);
      }
      $curGroup = ($(
          `<optgroup label="${verifier.type.classification.niceName}">
      `));
      currentOpt = verifier.type.classification.niceName;
    }
    if (verifierResults.verifier.length === 1) {
      $curGroup.append($(`
      <option value="${verifier.type.name}" selected>${verifier.type.niceName} (${verifier.size})</option>
    `));
    } else {
      $curGroup.append($(`
      <option value="${verifier.type.name}">${verifier.type.niceName} (${verifier.size})</option>
    `));
    }
  }
  $select.append($curGroup);
  $header.append($select);
  if (verifierResults.verifier.length === 1) {
    renderVerifierResult($select[0].selectedOptions[0]);
  }
}

/**
 *
 * @param verifier
 * @type VerifierResult
 */
function renderSelectEntry(verifier) {
  return $(`
      <option value="${verifier.type.name}">${verifier.type.niceName} (${verifier.size})</option>
    `);
}

/**
 *
 * @param verifierSelect
 * @Type option
 */
function renderVerifierResult(verifierSelect) {
  let verifier = findVerifierByName(verifierSelect.value);
  let $verifierEntries = $('#dmn-verifier-content');
  $verifierEntries.empty();
  if (verifier !== undefined) {
    $verifierEntries.append($(`
    <div>${verifier.type.niceName}: ${verifier.type.description}</div>
  `));
    let $entryList = $(`<ul>`);
    $verifierEntries.append($entryList);
    for (let i = 0; i < verifier.entries.length; i++) {
      $entryList.append(renderVerificationEntry(verifier.entries[i]));
    }
  }
}

/**
 *
 * @param verificationEntry
 * @Type VerificationEntry
 */
function renderVerificationEntry(verificationEntry) {
  console.log(verificationEntry);
  return $(`
    <li class="dmn-${verificationEntry.verificationClassification}">${verificationEntry.message}</li>
  `);
}

/**
 *
 * @param verifierName
 * @type string
 * @return VerifierResult
 */
function findVerifierByName(verifierName) {
  for (let i = 0; i < verifierResults.verifier.length; i++) {
    if (verifierResults.verifier[i].type.name === verifierName) {
      return verifierResults.verifier[i];
    }
  }
  return undefined;
}
